@@ -17,6 +17,7 @@ from DBUtils.PooledDB import PooledDB
 from dhkit import log
 from dhkit.threadpool import ThreadPool, Result
 from functools import partial, wraps
+from collections import OrderedDict
 
 
 class DatabaseError(Exception):
@@ -52,6 +53,38 @@ class ConnectionFactory(object):
         cls._thread_pool.stop()
 
 
+class MetaData(OrderedDict):
+
+    def __init__(self, desc_row):
+        super(MetaData, self).__init__()
+        self['name'] = desc_row[0]
+        self['type_code'] = desc_row[1]
+        self['display_size'] = desc_row[2]
+        self['internal_size'] = desc_row[3]
+        self['precision'] = desc_row[4]
+        self['scale'] = desc_row[5]
+        self['null_ok'] = desc_row[6]
+
+    def __getitem__(self, key):
+        try:
+            return super(MetaData, self).__getitem__(key)
+        except KeyError:
+            if isinstance(key, int):
+                name, value = self.items()[key]
+                return value
+            else:
+                raise
+        except TypeError:
+            if isinstance(key, slice):
+                items = self.items()[key]
+                values = []
+                for name, value in items:
+                    values.append(value)
+                return tuple(values)
+            else:
+                raise
+
+
 class RowMapper(object):
     """
     This is an interface to handle one row of data.
@@ -194,16 +227,7 @@ class DatabaseTemplate(object):
         rv = yield self.execute(sql_statement, params, conn)
         raise tornado.gen.Return(rv[0])
 
-    def _query(self, sql_statement, params=None, conn=None, row_mapper=None):
-
-        def get_metadata(desc_row):
-            return {"name": desc_row[0],
-                    "type_code": desc_row[1],
-                    "display_size": desc_row[2],
-                    "internal_size": desc_row[3],
-                    "precision": desc_row[4],
-                    "scale": desc_row[5],
-                    "null_ok": desc_row[6]}
+    def _query(self, sql_statement, params=None, conn=None, return_metadata=False, row_mapper=None):
         if conn is None:
             in_transaction = False
             conn = self.connection_pool.connection()
@@ -217,13 +241,17 @@ class DatabaseTemplate(object):
             else:
                 cursor.execute(sql_statement)
             rs = cursor.fetchall()
+            metadata = [MetaData(desc_row) for desc_row in cursor.description]
             if rs:
                 if not row_mapper:
                     row_mapper = self.ROWMAPPER_CLS()
-                metadata = [get_metadata(row) for row in cursor.description]
-                return [row_mapper.map_row(row, metadata) for row in rs]
+                result = [row_mapper.map_row(row, metadata) for row in rs]
             else:
-                return []
+                result = []
+            if return_metadata:
+                return result, metadata
+            else:
+                return result
         except Exception, e:
             log.exception("database query error. sql:[%s], params:[%s]" % (sql_statement, params))
             raise DatabaseError(self.fomat_exc(e))
@@ -236,12 +264,13 @@ class DatabaseTemplate(object):
                 log.debug("release database connection error: %s" % e.message)
 
     @async
-    def query(self, sql_statement, params=None, conn=None, row_mapper=None, callback=None):
-        self.threadpool.add_task(partial(self._query, sql_statement, params, conn, row_mapper), callback)
+    def query(self, sql_statement, params=None, conn=None, return_metadata=False, row_mapper=None, callback=None):
+        self.threadpool.add_task(
+            partial(self._query, sql_statement, params, conn, return_metadata, row_mapper), callback)
 
     @tornado.gen.coroutine
-    def get(self, sql_statement, params=None, conn=None, row_mapper=None):
-        rs = yield self.query(sql_statement, params, conn, row_mapper)
+    def get(self, sql_statement, params=None, conn=None, return_metadata=False, row_mapper=None):
+        rs = yield self.query(sql_statement, params, conn, return_metadata, row_mapper)
         raise tornado.gen.Return(rs[0] if rs else None)
 
     def fomat_exc(self, exc):
